@@ -3,7 +3,7 @@ from __future__ import absolute_import
 import socket
 import types
 
-from anyjson import dumps
+from anyjson import dumps, loads
 from collections import defaultdict
 from itertools import count
 
@@ -221,8 +221,36 @@ class Transport(redis.Transport):
 class test_Channel(Case):
 
     def setUp(self):
-        self.connection = Connection(transport=Transport)
-        self.channel = self.connection.channel()
+        self.connection = self.create_connection()
+        self.channel = self.connection.default_channel
+
+    def create_connection(self, **kwargs):
+        kwargs.setdefault('transport_options', {'fanout_patterns': True})
+        return Connection(transport=Transport, **kwargs)
+
+    def _get_one_delivery_tag(self, n='test_uniq_tag'):
+        with self.create_connection() as conn1:
+            chan = conn1.default_channel
+            chan.exchange_declare(n)
+            chan.queue_declare(n)
+            chan.queue_bind(n, n, n)
+            msg = chan.prepare_message('quick brown fox')
+            chan.basic_publish(msg, n, n)
+            q, payload = chan.client.brpop([n])
+            self.assertEqual(q, n)
+            self.assertTrue(payload)
+            pymsg = chan.message_to_python(loads(payload))
+            return pymsg.delivery_tag
+
+    def test_delivery_tag_is_uuid(self):
+        seen = set()
+        for i in range(100):
+            tag = self._get_one_delivery_tag()
+            self.assertNotIn(tag, seen)
+            seen.add(tag)
+            with self.assertRaises(ValueError):
+                int(tag)
+            self.assertEqual(len(tag), 36)
 
     def test_disable_ack_emulation(self):
         conn = Connection(transport=Transport, transport_options={
@@ -395,11 +423,11 @@ class test_Channel(Case):
         self.channel.subclient = Mock()
         self.channel.active_fanout_queues.add('a')
         self.channel.active_fanout_queues.add('b')
-        self.channel._fanout_queues.update(a='a', b='b')
+        self.channel._fanout_queues.update(a=('a', ''), b=('b', ''))
 
         self.channel._subscribe()
-        self.assertTrue(self.channel.subclient.subscribe.called)
-        s_args, _ = self.channel.subclient.subscribe.call_args
+        self.assertTrue(self.channel.subclient.psubscribe.called)
+        s_args, _ = self.channel.subclient.psubscribe.call_args
         self.assertItemsEqual(s_args[0], ['a', 'b'])
 
         self.channel.subclient.connection._sock = None
@@ -470,7 +498,7 @@ class test_Channel(Case):
 
     def test_receive_different_message_Type(self):
         s = self.channel.subclient = Mock()
-        s.parse_response.return_value = ['pmessage', '/foo/', 0, 'data']
+        s.parse_response.return_value = ['message', '/foo/', 0, 'data']
 
         with self.assertRaises(redis.Empty):
             self.channel._receive()
@@ -518,7 +546,7 @@ class test_Channel(Case):
         c = self.channel.client = Mock()
 
         body = {'hello': 'world'}
-        self.channel._put_fanout('exchange', body)
+        self.channel._put_fanout('exchange', body, '')
         c.publish.assert_called_with('exchange', dumps(body))
 
     def test_put_priority(self):
@@ -568,7 +596,7 @@ class test_Channel(Case):
         self.channel.close()
 
     def test_close_deletes_autodelete_fanout_queues(self):
-        self.channel._fanout_queues = ['foo', 'bar']
+        self.channel._fanout_queues = {'foo': ('foo', ''), 'bar': ('bar', '')}
         self.channel.auto_delete_queues = ['foo']
         self.channel.queue_delete = Mock(name='queue_delete')
 
